@@ -5,6 +5,8 @@
 {-# LANGUAGE DeriveTraversable #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE KindSignatures #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE StandaloneDeriving #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE TypeOperators #-}
@@ -34,7 +36,7 @@ module Data.Orphans () where
 import           Control.Monad.ST as Strict
 #endif
 
-#if __GLASGOW_HASKELL__ >= 701 && __GLASGOW_HASKELL__ < 710
+#if __GLASGOW_HASKELL__ >= 701 && !(MIN_VERSION_base(4,9,0))
 import           GHC.Generics as Generics
 #endif
 
@@ -42,25 +44,28 @@ import           GHC.Generics as Generics
 import           Control.Monad.Instances ()
 #endif
 
-#if __GLASGOW_HASKELL__ < 710
-import           Control.Exception as Exception
-import           Control.Monad.ST.Lazy as Lazy
+#if !(MIN_VERSION_base(4,9,0))
 import           Data.Data as Data
 import qualified Data.Foldable as F (Foldable(..))
 import           Data.Monoid as Monoid
 import qualified Data.Traversable as T (Traversable(..))
+import           Text.ParserCombinators.ReadPrec as ReadPrec
+import           Text.Read as Read
+#endif
+
+#if __GLASGOW_HASKELL__ < 710
+import           Control.Exception as Exception
+import           Control.Monad.ST.Lazy as Lazy
 import           GHC.Exts as Exts
 import           GHC.IO.Exception as Exception
 import           Text.ParserCombinators.ReadP as ReadP
-import           Text.ParserCombinators.ReadPrec as ReadPrec
-import           Text.Read as Read
 
 # if defined(mingw32_HOST_OS)
 import           GHC.ConsoleHandler as Console
 # endif
 #endif
 
-#if !(MIN_VERSION_base(4,8,0))
+#if !(MIN_VERSION_base(4,9,0))
 import           Data.Orphans.Prelude
 #endif
 
@@ -366,8 +371,8 @@ instance Storable a
 instance (Storable a, RealFloat a)
 # endif
   => Storable (Complex a) where
-    sizeOf (a :+ _)    = 2 * sizeOf a
-    alignment (a :+ _) = alignment a
+    sizeOf a       = 2 * sizeOf (realPart a)
+    alignment a    = alignment (realPart a)
     peek p           = do
                         q <- return $ castPtr p
                         r <- peek q
@@ -379,8 +384,8 @@ instance (Storable a, RealFloat a)
                         pokeElemOff q 1 i
 
 instance (Storable a, Integral a) => Storable (Ratio a) where
-    sizeOf (n :% _)    = 2 * sizeOf n
-    alignment (n :% _) = alignment n
+    sizeOf _    = 2 * sizeOf (undefined :: a)
+    alignment _ = alignment (undefined :: a )
     peek p           = do
                         q <- return $ castPtr p
                         r <- peek q
@@ -392,6 +397,505 @@ instance (Storable a, Integral a) => Storable (Ratio a) where
                         pokeElemOff q 1 i
 #endif
 
+#if !(MIN_VERSION_base(4,9,0))
+instance Storable () where
+  sizeOf _ = 0
+  alignment _ = 1
+  peek _ = return ()
+  poke _ _ = return ()
+
+deriving instance Bits a       => Bits (Const a b)
+deriving instance Bounded a    => Bounded (Const a b)
+deriving instance Enum a       => Enum (Const a b)
+deriving instance Floating a   => Floating (Const a b)
+deriving instance Fractional a => Fractional (Const a b)
+deriving instance Integral a   => Integral (Const a b)
+deriving instance IsString a   => IsString (Const a b)
+deriving instance Ix a         => Ix (Const a b)
+deriving instance Num a        => Num (Const a b)
+deriving instance Real a       => Real (Const a b)
+deriving instance RealFloat a  => RealFloat (Const a b)
+deriving instance RealFrac a   => RealFrac (Const a b)
+deriving instance Storable a   => Storable (Const a b)
+
+deriving instance           Data All
+deriving instance           Data Monoid.Any
+deriving instance Data a => Data (Dual a)
+deriving instance Data a => Data (First a)
+deriving instance Data a => Data (Last a)
+deriving instance Data a => Data (Product a)
+deriving instance Data a => Data (Sum a)
+
+instance F.Foldable Dual where
+    foldMap            = coerce
+
+    foldl              = coerce
+    foldl1 _           = getDual
+    foldr f z (Dual x) = f x z
+    foldr1 _           = getDual
+# if MIN_VERSION_base(4,6,0)
+    foldl'             = coerce
+    foldr'             = F.foldr
+# endif
+# if MIN_VERSION_base(4,8,0)
+    elem               = (. getDual) #. (==)
+    length _           = 1
+    maximum            = getDual
+    minimum            = getDual
+    null _             = False
+    product            = getDual
+    sum                = getDual
+    toList (Dual x)    = [x]
+# endif
+
+instance F.Foldable Sum where
+    foldMap            = coerce
+
+    foldl              = coerce
+    foldl1 _           = getSum
+    foldr f z (Sum x)  = f x z
+    foldr1 _           = getSum
+# if MIN_VERSION_base(4,6,0)
+    foldl'                = coerce
+    foldr'                = F.foldr
+# endif
+# if MIN_VERSION_base(4,8,0)
+    elem               = (. getSum) #. (==)
+    length _           = 1
+    maximum            = getSum
+    minimum            = getSum
+    null _             = False
+    product            = getSum
+    sum                = getSum
+    toList (Sum x)     = [x]
+# endif
+
+instance F.Foldable Product where
+    foldMap               = coerce
+
+    foldl                 = coerce
+    foldl1 _              = getProduct
+    foldr f z (Product x) = f x z
+    foldr1 _              = getProduct
+# if MIN_VERSION_base(4,6,0)
+    foldl'                = coerce
+    foldr'                = F.foldr
+# endif
+# if MIN_VERSION_base(4,8,0)
+    elem                  = (. getProduct) #. (==)
+    length _              = 1
+    maximum               = getProduct
+    minimum               = getProduct
+    null _                = False
+    product               = getProduct
+    sum                   = getProduct
+    toList (Product x)    = [x]
+# endif
+
+# if MIN_VERSION_base(4,8,0)
+(#.)   :: Coercible b c => (b -> c) -> (a -> b) -> (a -> c)
+(#.) _f = coerce
+# endif
+
+# if !(MIN_VERSION_base(4,7,0))
+coerce ::                  a -> b
+coerce = unsafeCoerce
+# endif
+
+instance Functor Dual where
+    fmap     = coerce
+
+instance Applicative Dual where
+    pure     = Dual
+    (<*>)    = coerce
+
+instance Monad Dual where
+    return   = Dual
+    m >>= k  = k (getDual m)
+
+instance Functor Sum where
+    fmap     = coerce
+
+instance Applicative Sum where
+    pure     = Sum
+    (<*>)    = coerce
+
+instance Monad Sum where
+    return   = Sum
+    m >>= k  = k (getSum m)
+
+instance Functor Product where
+    fmap     = coerce
+
+instance Applicative Product where
+    pure     = Product
+    (<*>)    = coerce
+
+instance Monad Product where
+    return   = Product
+    m >>= k  = k (getProduct m)
+
+instance F.Foldable First where
+    foldMap f = F.foldMap f . getFirst
+
+instance F.Foldable Last where
+    foldMap f = F.foldMap f . getLast
+
+instance Monoid a => Monoid (IO a) where
+    mempty = pure mempty
+    mappend = liftA2 mappend
+
+-- see: #10190 https://git.haskell.org/ghc.git/commitdiff/9db005a444722e31aca1956b058e069bcf3cacbd
+instance Monoid a => Monad ((,) a) where
+    return x = (mempty, x)
+    (u, a) >>= k = case k a of (v, b) -> (u `mappend` v, b)
+
+instance MonadFix Dual where
+    mfix f   = Dual (fix (getDual . f))
+
+instance MonadFix Sum where
+    mfix f   = Sum (fix (getSum . f))
+
+instance MonadFix Product where
+    mfix f   = Product (fix (getProduct . f))
+
+instance MonadFix First where
+    mfix f   = First (mfix (getFirst . f))
+
+instance MonadFix Last where
+    mfix f   = Last (mfix (getLast . f))
+
+instance T.Traversable Dual where
+    traverse f (Dual x) = Dual <$> f x
+
+instance T.Traversable Sum where
+    traverse f (Sum x) = Sum <$> f x
+
+instance T.Traversable Product where
+    traverse f (Product x) = Product <$> f x
+
+instance T.Traversable First where
+    traverse f (First x) = First <$> T.traverse f x
+
+instance T.Traversable Last where
+    traverse f (Last x) = Last <$> T.traverse f x
+
+deriving instance F.Foldable    ZipList
+deriving instance T.Traversable ZipList
+
+# if MIN_VERSION_base(4,4,0)
+deriving instance Functor     Complex
+deriving instance F.Foldable    Complex
+deriving instance T.Traversable Complex
+
+instance Applicative Complex where
+  pure a = a :+ a
+  f :+ g <*> a :+ b = f a :+ g b
+
+instance Monad Complex where
+  return a = a :+ a
+  a :+ b >>= f = realPart (f a) :+ imagPart (f b)
+
+instance MonadZip Dual where
+    -- Cannot use coerce, it's unsafe
+    mzipWith = liftM2
+
+instance MonadZip Sum where
+    mzipWith = liftM2
+
+instance MonadZip Product where
+    mzipWith = liftM2
+
+instance MonadZip Maybe where
+    mzipWith = liftM2
+
+instance MonadZip First where
+    mzipWith = liftM2
+
+instance MonadZip Last where
+    mzipWith = liftM2
+# endif
+
+# if MIN_VERSION_base(4,7,0)
+instance Alternative Proxy where
+    empty = Proxy
+    {-# INLINE empty #-}
+    _ <|> _ = Proxy
+    {-# INLINE (<|>) #-}
+
+instance MonadPlus Proxy where
+#  if !(MIN_VERSION_base(4,8,0))
+    mzero = Proxy
+    {-# INLINE mzero #-}
+    mplus _ _ = Proxy
+    {-# INLINE mplus #-}
+#  endif
+
+instance MonadZip Proxy where
+    mzipWith _ _ _ = Proxy
+    {-# INLINE mzipWith #-}
+
+deriving instance FiniteBits a => FiniteBits (Const a b)
+# endif
+
+# if MIN_VERSION_base(4,8,0)
+deriving instance (Data (f a), Typeable f, Typeable a)
+    => Data (Alt (f :: * -> *) (a :: *))
+
+instance MonadFix f => MonadFix (Alt f) where
+    mfix f   = Alt (mfix (getAlt . f))
+
+instance MonadZip f => MonadZip (Alt f) where
+    mzipWith f (Alt ma) (Alt mb) = Alt (mzipWith f ma mb)
+
+deriving instance Bits a       => Bits (Identity a)
+deriving instance Bounded a    => Bounded (Identity a)
+deriving instance Enum a       => Enum (Identity a)
+deriving instance FiniteBits a => FiniteBits (Identity a)
+deriving instance Floating a   => Floating (Identity a)
+deriving instance Fractional a => Fractional (Identity a)
+deriving instance Integral a   => Integral (Identity a)
+deriving instance IsString a   => IsString (Identity a)
+deriving instance Ix a         => Ix (Identity a)
+deriving instance Monoid a     => Monoid (Identity a)
+deriving instance Num a        => Num (Identity a)
+deriving instance Real a       => Real (Identity a)
+deriving instance RealFloat a  => RealFloat (Identity a)
+deriving instance RealFrac a   => RealFrac (Identity a)
+deriving instance Storable a   => Storable (Identity a)
+# endif
+
+# if __GLASGOW_HASKELL__ >= 701
+deriving instance Data p => Data (V1 p)
+deriving instance Data p => Data (U1 p)
+deriving instance Data p => Data (Par1 p)
+deriving instance (Typeable i, Data p, Data c) => Data (K1 i c p)
+deriving instance Data Generics.Fixity
+deriving instance Data Associativity
+
+deriving instance                                 F.Foldable V1
+deriving instance                                 F.Foldable Par1
+deriving instance F.Foldable f                 => F.Foldable (Rec1 f)
+deriving instance                                 F.Foldable (K1 i c)
+deriving instance F.Foldable f                 => F.Foldable (M1 i c f)
+deriving instance (F.Foldable f, F.Foldable g) => F.Foldable (f :+: g)
+deriving instance (F.Foldable f, F.Foldable g) => F.Foldable (f :*: g)
+deriving instance (F.Foldable f, F.Foldable g) => F.Foldable (f :.: g)
+
+deriving instance                           Functor V1
+deriving instance                           Functor Par1
+deriving instance Functor f              => Functor (Rec1 f)
+deriving instance                           Functor (K1 i c)
+deriving instance Functor f              => Functor (M1 i c f)
+deriving instance (Functor f, Functor g) => Functor (f :+: g)
+deriving instance (Functor f, Functor g) => Functor (f :*: g)
+deriving instance (Functor f, Functor g) => Functor (f :.: g)
+
+instance MonadFix Par1 where
+    mfix f = Par1 (fix (unPar1 . f))
+
+instance MonadFix f => MonadFix (Rec1 f) where
+    mfix f = Rec1 (mfix (unRec1 . f))
+
+instance MonadFix f => MonadFix (M1 i c f) where
+    mfix f = M1 (mfix (unM1. f))
+
+instance (MonadFix f, MonadFix g) => MonadFix (f :*: g) where
+    mfix f = (mfix (fstP . f)) :*: (mfix (sndP . f))
+      where
+        fstP (a :*: _) = a
+        sndP (_ :*: b) = b
+
+instance MonadZip U1 where
+    mzipWith _ _ _ = U1
+
+instance MonadZip Par1 where
+    mzipWith = liftM2
+
+instance MonadZip f => MonadZip (Rec1 f) where
+    mzipWith f (Rec1 fa) (Rec1 fb) = Rec1 (mzipWith f fa fb)
+
+instance MonadZip f => MonadZip (M1 i c f) where
+    mzipWith f (M1 fa) (M1 fb) = M1 (mzipWith f fa fb)
+
+instance (MonadZip f, MonadZip g) => MonadZip (f :*: g) where
+    mzipWith f (x1 :*: y1) (x2 :*: y2) = mzipWith f x1 x2 :*: mzipWith f y1 y2
+
+deriving instance                                       T.Traversable V1
+deriving instance                                       T.Traversable Par1
+deriving instance T.Traversable f                    => T.Traversable (Rec1 f)
+deriving instance                                       T.Traversable (K1 i c)
+deriving instance T.Traversable f                    => T.Traversable (M1 i c f)
+deriving instance (T.Traversable f, T.Traversable g) => T.Traversable (f :+: g)
+deriving instance (T.Traversable f, T.Traversable g) => T.Traversable (f :*: g)
+deriving instance (T.Traversable f, T.Traversable g) => T.Traversable (f :.: g)
+
+deriving instance Bounded Associativity
+deriving instance Enum    Associativity
+deriving instance Ix      Associativity
+
+deriving instance Eq   (V1 p)
+deriving instance Ord  (V1 p)
+-- Implement Read instance manually to get around an old GHC bug
+-- (Trac #7931)
+instance Read (V1 p) where
+    readPrec     = parens ReadPrec.pfail
+    readList     = readListDefault
+    readListPrec = readListPrecDefault
+deriving instance Show (V1 p)
+
+instance Functor U1 where
+  fmap _ _ = U1
+
+instance Applicative U1 where
+  pure _ = U1
+  _ <*> _ = U1
+
+instance Alternative U1 where
+  empty = U1
+  _ <|> _ = U1
+
+instance Monad U1 where
+#  if !(MIN_VERSION_base(4,8,0))
+  return _ = U1
+#  endif
+  _ >>= _ = U1
+
+instance MonadPlus U1 where
+#  if !(MIN_VERSION_base(4,8,0))
+  mzero = U1
+  mplus _ _ = U1
+#  endif
+
+instance F.Foldable U1 where
+    foldMap _ _ = mempty
+    {-# INLINE foldMap #-}
+    fold _ = mempty
+    {-# INLINE fold #-}
+    foldr _ z _ = z
+    {-# INLINE foldr #-}
+    foldl _ z _ = z
+    {-# INLINE foldl #-}
+    foldl1 _ _ = error "foldl1: U1"
+    foldr1 _ _ = error "foldr1: U1"
+#  if MIN_VERSION_base(4,8,0)
+    length _   = 0
+    null _     = True
+    elem _ _   = False
+    sum _      = 0
+    product _  = 1
+#  endif
+
+instance T.Traversable U1 where
+    traverse _ _ = pure U1
+    {-# INLINE traverse #-}
+    sequenceA _ = pure U1
+    {-# INLINE sequenceA #-}
+    mapM _ _ = return U1
+    {-# INLINE mapM #-}
+    sequence _ = return U1
+    {-# INLINE sequence #-}
+
+instance Applicative Par1 where
+  pure a = Par1 a
+  Par1 f <*> Par1 x = Par1 (f x)
+
+instance Monad Par1 where
+#  if !(MIN_VERSION_base(4,8,0))
+  return a = Par1 a
+#  endif
+  Par1 x >>= f = f x
+
+instance Applicative f => Applicative (Rec1 f) where
+  pure a = Rec1 (pure a)
+  Rec1 f <*> Rec1 x = Rec1 (f <*> x)
+
+instance Alternative f => Alternative (Rec1 f) where
+  empty = Rec1 empty
+  Rec1 l <|> Rec1 r = Rec1 (l <|> r)
+
+instance Monad f => Monad (Rec1 f) where
+#  if !(MIN_VERSION_base(4,8,0))
+  return a = Rec1 (return a)
+#  endif
+  Rec1 x >>= f = Rec1 (x >>= \a -> unRec1 (f a))
+
+instance MonadPlus f => MonadPlus (Rec1 f) where
+#  if !(MIN_VERSION_base(4,8,0))
+  mzero = Rec1 mzero
+  mplus (Rec1 a) (Rec1 b) = Rec1 (mplus a b)
+#  endif
+
+instance Applicative f => Applicative (M1 i c f) where
+  pure a = M1 (pure a)
+  M1 f <*> M1 x = M1 (f <*> x)
+
+instance Alternative f => Alternative (M1 i c f) where
+  empty = M1 empty
+  M1 l <|> M1 r = M1 (l <|> r)
+
+instance Monad f => Monad (M1 i c f) where
+#  if !(MIN_VERSION_base(4,8,0))
+  return a = M1 (return a)
+#  endif
+  M1 x >>= f = M1 (x >>= \a -> unM1 (f a))
+
+instance MonadPlus f => MonadPlus (M1 i c f) where
+#  if !(MIN_VERSION_base(4,8,0))
+  mzero = M1 mzero
+  mplus (M1 a) (M1 b) = M1 (mplus a b)
+#  endif
+
+instance (Applicative f, Applicative g) => Applicative (f :*: g) where
+  pure a = pure a :*: pure a
+  (f :*: g) <*> (x :*: y) = (f <*> x) :*: (g <*> y)
+
+instance (Alternative f, Alternative g) => Alternative (f :*: g) where
+  empty = empty :*: empty
+  (x1 :*: y1) <|> (x2 :*: y2) = (x1 <|> x2) :*: (y1 <|> y2)
+
+instance (Monad f, Monad g) => Monad (f :*: g) where
+#  if !(MIN_VERSION_base(4,8,0))
+  return a = return a :*: return a
+#  endif
+  (m :*: n) >>= f = (m >>= \a -> fstP (f a)) :*: (n >>= \a -> sndP (f a))
+    where
+      fstP (a :*: _) = a
+      sndP (_ :*: b) = b
+
+instance (MonadPlus f, MonadPlus g) => MonadPlus (f :*: g) where
+#  if !(MIN_VERSION_base(4,8,0))
+  mzero = mzero :*: mzero
+  (x1 :*: y1) `mplus` (x2 :*: y2) =  (x1 `mplus` x2) :*: (y1 `mplus` y2)
+#  endif
+
+instance (Applicative f, Applicative g) => Applicative (f :.: g) where
+  pure x = Comp1 (pure (pure x))
+  Comp1 f <*> Comp1 x = Comp1 (fmap (<*>) f <*> x)
+
+instance (Alternative f, Applicative g) => Alternative (f :.: g) where
+  empty = Comp1 empty
+  Comp1 x <|> Comp1 y = Comp1 (x <|> y)
+
+#  if MIN_VERSION_base(4,7,0)
+deriving instance (Data (f p), Typeable f, Data p) => Data (Rec1 f p)
+deriving instance (Data p, Data (f p), Typeable c, Typeable i, Typeable f)
+  => Data (M1 i c f p)
+deriving instance (Typeable f, Typeable g, Data p, Data (f p), Data (g p))
+  => Data ((f :+: g) p)
+deriving instance (Typeable f, Typeable g, Data p, Data (f p), Data (g p))
+  => Data ((f :*: g) p)
+deriving instance (Typeable f, Typeable g, Data p, Data (f (g p)))
+  => Data ((f :.: g) p)
+#  endif
+
+#  if MIN_VERSION_base(4,8,0)
+instance Bifunctor (K1 i) where
+    bimap f _ (K1 c) = K1 (f c)
+#  endif
+# endif
+#endif
+
 #if __GLASGOW_HASKELL__ < 710
 deriving instance Typeable  All
 deriving instance Typeable  AnnotationWrapper
